//  Dialog.cpp version 1.5
//  yudit package - Unicode Editor for the X Window System (and Linux) 
//
//  Author: gsinai@iname.com (Gaspar Sinai)
//  GNU Copyright (C) 1997,1998,1999  Gaspar Sinai
// 
//  yudit version 1.5  Copyright(C) 30 November, 1999, Tokyo Japan  Gaspar Sinai
//  yudit version 1.4  Copyright(C) 25 November, 1999, Tokyo Japan  Gaspar Sinai
//  yudit version 1.3  Copyright(C)  5 April,    1999, Tokyo Japan  Gaspar Sinai
//  yudit version 1.2  Copyright(C) 10 December, 1998, Tokyo Japan  Gaspar Sinai
//  yudit version 1.1  Copyright(C) 23 August,   1998, Tokyo Japan  Gaspar Sinai
//  yudit version 1.0  Copyright(C) 17 May,      1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.99 Copyright(C)  4 April,    1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.97 Copyright(C)  4 February, 1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.95 Copyright(C) 10 January,  1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.94 Copyright(C) 17 December, 1997, Tokyo Japan  Gaspar Sinai
//  yudit version 0.9 Copyright (C)  8 December, 1997, Tokyo Japan  Gaspar Sinai
//  yutex version 0.8 Copyright (C)  5 November, 1997, Tokyo Japan  Gaspar Sinai
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

#include <Xm/Xm.h>
#include <Xm/DialogS.h>
#include <Xm/FileSB.h>
#include <Xm/MessageB.h>
#include <Xm/SelectioB.h>

#include "Dialog.h"
#include "MYudit.h"
#include <UCommon.h>
#include <Translate.h>

Dialog::Dialog (
		Widget		parent,
		const char* 	titleIn,
		Button		buttons,
		DialogType	dialogTypeIn)
{
	dialog = 0;
	lastString = 0;
	lastButton = HELP;
	title = 0;
	XmString 	titleString;
	XmString	labelString;
	char*		tmpStr;
	const char* 	str;

	dialogType = dialogTypeIn;

	title = new char[strlen (titleIn) +1];
	CHECKNULL (title);
	strcpy (title, titleIn);

	// see if we are going to create that dialog...
	str = Translate (title);
	tmpStr = new char[strlen (str) +1];
	CHECKNULL (tmpStr);
	strcpy (tmpStr, str);
	titleString = XmStringCreateLtoR (tmpStr,
		XmSTRING_DEFAULT_CHARSET);
	delete tmpStr;
	switch (dialogTypeIn)
	{
	case DFILE_READ:
		dialog = XmCreateFileSelectionDialog (parent, title, 0, 0);
		XtVaSetValues (dialog,
			XmNdialogStyle,	XmDIALOG_PRIMARY_APPLICATION_MODAL,
			XmNdialogTitle, titleString, 
			XmNmustMatch,	True,
			XmNautoUnmanage, True,
			0);
		break;
	case DFILE_WRITE:
		dialog = XmCreateFileSelectionDialog (parent, title, 0, 0);
		XtVaSetValues (dialog,
			XmNdialogStyle,	XmDIALOG_PRIMARY_APPLICATION_MODAL,
			XmNdialogTitle, titleString, 
			XmNautoUnmanage, True,
			0);
		break;
	case DPROMPT:
		dialog = XmCreatePromptDialog (parent, title, 0, 0);
		XtVaSetValues (dialog,
			XmNdialogStyle,	XmDIALOG_PRIMARY_APPLICATION_MODAL,
			XmNdialogTitle, titleString, 
			XmNautoUnmanage, True,
			0);
		break;
	case DWARNING:
	case DQUESTION:
	case DINFO:
	default:
		dialog = XmCreateQuestionDialog (parent, title, 0, 0);
		XtVaSetValues (dialog,
			XmNdialogStyle,	XmDIALOG_PRIMARY_APPLICATION_MODAL,
			XmNdialogTitle, titleString, 
			XmNautoUnmanage, True,
			0);
		XtVaSetValues (XmMessageBoxGetChild (
			  dialog, XmDIALOG_MESSAGE_LABEL),
			XmNalignment, XmALIGNMENT_CENTER, 0);
	}
	XmStringFree (titleString);
	switch (dialogTypeIn)
	{
	case DFILE_READ:
	case DFILE_WRITE:
		if ((buttons&HELP)==0)
		{
			XtUnmanageChild (XmFileSelectionBoxGetChild (dialog, 
				XmDIALOG_HELP_BUTTON));
		}
		else
		{
			str = Translate ("Help");
			tmpStr = new char[strlen (str) +1];
			CHECKNULL (tmpStr);
			strcpy (tmpStr, str);
			labelString = XmStringCreateLtoR (tmpStr,
				XmSTRING_DEFAULT_CHARSET);
			delete tmpStr;
			XtVaSetValues (dialog, 
				XmNhelpLabelString, labelString, 0);
			XmStringFree (labelString);
		}
		if ((buttons&CANCEL)==0)
		{
			XtUnmanageChild (XmFileSelectionBoxGetChild (dialog, 
				XmDIALOG_CANCEL_BUTTON));
		}
		else
		{
			str = Translate ("Cancel");
			tmpStr = new char[strlen (str) +1];
			CHECKNULL (tmpStr);
			strcpy (tmpStr, str);
			labelString = XmStringCreateLtoR (tmpStr,
				XmSTRING_DEFAULT_CHARSET);
			delete tmpStr;
			XtVaSetValues (dialog, 
				XmNcancelLabelString, labelString, 0);
			XmStringFree (labelString);
		}
		if ((buttons&OK)==0)
		{
			XtUnmanageChild (XmFileSelectionBoxGetChild (dialog, 
				XmDIALOG_OK_BUTTON));
		}
		else
		{
			str = Translate ("OK");
			tmpStr = new char[strlen (str) +1];
			CHECKNULL (tmpStr);
			strcpy (tmpStr, str);
			labelString = XmStringCreateLtoR (tmpStr,
				XmSTRING_DEFAULT_CHARSET);
			delete tmpStr;
			XtVaSetValues (dialog, 
				XmNhelpLabelString, labelString, 0);
			XmStringFree (labelString);
		}
		break;
	case DPROMPT:
		if ((buttons&HELP)==0)
		{
			XtUnmanageChild (XmSelectionBoxGetChild (dialog, 
				XmDIALOG_HELP_BUTTON));
		}
		else
		{
			str = Translate ("Help");
			tmpStr = new char[strlen (str) +1];
			CHECKNULL (tmpStr);
			strcpy (tmpStr, str);
			labelString = XmStringCreateLtoR (tmpStr,
				XmSTRING_DEFAULT_CHARSET);
			delete tmpStr;
			XtVaSetValues (dialog, 
				XmNhelpLabelString, labelString, 0);
			XmStringFree (labelString);
		}
		if ((buttons&CANCEL)==0)
		{
			XtUnmanageChild (XmSelectionBoxGetChild (dialog, 
				XmDIALOG_CANCEL_BUTTON));
		}
		else
		{
			str = Translate ("Cancel");
			tmpStr = new char[strlen (str) +1];
			CHECKNULL (tmpStr);
			strcpy (tmpStr, str);
			labelString = XmStringCreateLtoR (tmpStr,
				XmSTRING_DEFAULT_CHARSET);
			delete tmpStr;
			XtVaSetValues (dialog, 
				XmNcancelLabelString, labelString, 0);
			XmStringFree (labelString);
		}
		if ((buttons&OK)==0)
		{
			XtUnmanageChild (XmSelectionBoxGetChild (dialog, 
				XmDIALOG_OK_BUTTON));
		}
		else
		{
			str = Translate ("OK");
			tmpStr = new char[strlen (str) +1];
			CHECKNULL (tmpStr);
			strcpy (tmpStr, str);
			labelString = XmStringCreateLtoR (tmpStr,
				XmSTRING_DEFAULT_CHARSET);
			delete tmpStr;
			XtVaSetValues (dialog, 
				XmNokLabelString, labelString, 0);
			XmStringFree (labelString);
		}
		break;
	case DWARNING:
	case DQUESTION:
	case DINFO:
		if ((buttons&HELP)==0)
		{
			XtUnmanageChild (XmMessageBoxGetChild (dialog, 
				XmDIALOG_HELP_BUTTON));
		}
		else
		{
			str = Translate ("Help");
			tmpStr = new char[strlen (str) +1];
			CHECKNULL (tmpStr);
			strcpy (tmpStr, str);
			labelString = XmStringCreateLtoR (tmpStr,
				XmSTRING_DEFAULT_CHARSET);
			delete tmpStr;
			XtVaSetValues (dialog, 
				XmNhelpLabelString, labelString, 0);
			XmStringFree (labelString);
		}
		if ((buttons&CANCEL)==0)
		{
			XtUnmanageChild (XmMessageBoxGetChild (dialog, 
				XmDIALOG_CANCEL_BUTTON));
		}
		else
		{
			str = Translate ("Cancel");
			tmpStr = new char[strlen (str) +1];
			CHECKNULL (tmpStr);
			strcpy (tmpStr, str);
			labelString = XmStringCreateLtoR (tmpStr,
				XmSTRING_DEFAULT_CHARSET);
			delete tmpStr;
			XtVaSetValues (dialog, 
				XmNcancelLabelString, labelString, 0);
			XmStringFree (labelString);
		}
		if ((buttons&OK)==0)
		{
			XtUnmanageChild (XmMessageBoxGetChild (dialog, 
				XmDIALOG_OK_BUTTON));
		}
		else
		{
			str = Translate ("OK");
			tmpStr = new char[strlen (str) +1];
			CHECKNULL (tmpStr);
			strcpy (tmpStr, str);
			labelString = XmStringCreateLtoR (tmpStr,
				XmSTRING_DEFAULT_CHARSET);
			delete tmpStr;
			XtVaSetValues (dialog, 
				XmNokLabelString, labelString, 0);
			XmStringFree (labelString);
		}
		break;
	}
	switch (dialogTypeIn)
	{
	case DWARNING:
		XtVaSetValues (dialog, XmNdialogType, XmDIALOG_WARNING, 0);
		break;
	case DQUESTION:
		XtVaSetValues (dialog, XmNdialogType, XmDIALOG_QUESTION, 0);
		break;
	case DINFO:
		XtVaSetValues (dialog, XmNdialogType, XmDIALOG_INFORMATION, 0);
		break;
	default:
		break;
	}

	XtAddCallback (dialog, XmNokCallback, 
		(XtCallbackProc) MYuditMessageCallback, this);
	XtAddCallback (dialog, XmNcancelCallback, 
		(XtCallbackProc) MYuditMessageCallback, this);
}

Dialog::~Dialog()
{
	if (dialog != 0) XtDestroyWidget (dialog);
	if (lastString !=0) XtFree (lastString);
	if (title !=0) delete title;
}

Dialog::Button
Dialog::getButton ()
{
	lastButton = HELP;
	if (lastString != 0)
	{
		XtFree (lastString);
		lastString=0;
	}
	XtManageChild (dialog);
	XtPopup (XtParent (dialog), XtGrabNone);
	MWait4Confirm (dialog);
	return lastButton;
}

void
Dialog::setMessage (const char* label)
{

	char*		tmpStr;
	const char* 	str;

	XmString labelString;

	str = Translate (label);
	tmpStr = new char[strlen (str) +1];
	CHECKNULL (tmpStr);
	strcpy (tmpStr, str);
	labelString = XmStringCreateLtoR (tmpStr,
		XmSTRING_DEFAULT_CHARSET);
	delete tmpStr;
	
	switch (dialogType)
	{
	case DPROMPT:
		XtVaSetValues (dialog,
			XmNdialogStyle,	XmDIALOG_PRIMARY_APPLICATION_MODAL,
			XmNselectionLabelString, labelString, 
			0);
		break;
	case DWARNING:
	case DQUESTION:
	case DINFO:
		XtVaSetValues (dialog,
			XmNdialogStyle,	XmDIALOG_PRIMARY_APPLICATION_MODAL,
			XmNmessageString, labelString, 
			0);
		break;
	default:
		break;
	}
	XmStringFree (labelString);
}

const char*
Dialog::getString ()
{
	XmString	str;

	if (lastString!=0) return lastString;

	XtVaGetValues (dialog, XmNtextString, &str, 0);
	XmStringGetLtoR (str, XmSTRING_DEFAULT_CHARSET, &lastString);
	XmStringFree (str);
	return lastString;
}

void
MYuditMessageCallback (Widget w, Dialog* dialog, XmAnyCallbackStruct *cbs)
{
	dialog->lastButton=Dialog::HELP;
	if (cbs->reason == XmCR_OK)
	{
		dialog->lastButton=Dialog::OK;
	}
	else if (cbs->reason == XmCR_CANCEL)
	{
		dialog->lastButton=Dialog::CANCEL;
	}
}

